@@ -3,10 +3,15 @@ package Hades;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 use Module::Generate;
 use Switch::Again qw/switch/;
 
+our $PARENTHESES;
+BEGIN {
+	$PARENTHESES = qr{ \( (?: (?> [^()]+ ) | (??{ $PARENTHESES }) )* \) }x;
+}
+
 sub new {
 	my ($class, %args) = (shift, scalar @_ == 1 ? %{$_[0]} : @_);
 	$args{macros} = {} if !$args{macros};
@@ -101,15 +106,15 @@ sub run {
 
 sub build_class {
 	my ($self, $mg, $class) = @_;
+	while ($class->[0] =~ m/^(dist|lib|tlib|realm|author|email|version)$/) {
+		$mg->$1($class->[1]);
+		shift @{$class}, shift @{$class};
+	}
 	if ($class->[0] eq 'macro') {
 		shift @{$class};
 		$self->build_macro($mg, $class);
 		return [];
 	}
-	while ($class->[0] =~ m/^(dist|lib|tlib|realm|author|email|version)$/) {
-		$mg->$1($class->[1]);
-		shift @{$class}, shift @{$class};
-	}
 	$mg->class(shift @{$class})->new;
 	return $class;
 }
@@ -122,7 +127,7 @@ sub build_new {
 		$accessors .= qq|$_ => {|;
 		$accessors .= qq|required => 1,| if $meta->{$_}->{required};
 		$accessors .= qq|default => $meta->{$_}->{default},| if $meta->{$_}->{default};
-		$accessors .= qq|builder => sub { my (\$self, \$value) = \@_;| . $self->build_builder($_, '$value', $meta->{$_}->{builder}) . qq|return \$value;}| 
+		$accessors .= qq|builder => sub { my (\$self, \$value) = \@_;| . $self->build_builder($_, '$value', $meta->{$_}->{builder}) . qq|return \$value;}|
 			if $meta->{$_}->{builder};
 		$accessors .= qq|},|;
 	} grep { $meta->{$_}->{meta} eq 'ACCESSOR' } keys %{$meta};
@@ -259,6 +264,9 @@ sub build_sub_or_accessor_attributes {
 			$value =~ s/^\:example\(\s*(.*)\s*\)$/$1/sg;
 			$meta->{$name}->{example} =  $value;
 		},
+		qr/^(\:no_success_test)/ => sub {
+			$meta->{$name}->{no_success_test} = 1;
+		},
 		qr/^(\:pod)/ => sub {
 			my $value = shift;
 			$value =~ s/^:pod\(\s*(.*)\s*\)$/$1/sg;
@@ -422,7 +430,7 @@ sub build_sub {
 sub build_code {
 	my ($self, $mg, $name, $code) = @_;
 	return unless defined $code;
-	$code =~ s/€(.*);/$self->build_macro_code($mg, $1)/eg;
+	1 while $code =~ s/€(\w+(|$PARENTHESES));/$self->build_macro_code($mg, $1)/ge;
 	$code =~ s/£(\w*(\s|\$|\-|\;|\,|\{|\}|\[|\]|\)|\(|\:))/$self->build_self($1)/eg;
 	return $code;
 }
@@ -434,12 +442,13 @@ sub build_self {
 
 sub build_macro_code {
 	my ($self, $mg, $match) = @_;
-	if ($match =~ m/^(.*)\((.*)\)$/) {
+	if ($match =~ m/^(.*)($PARENTHESES)$/m) {
 		return '' unless $self->{macros}->{$1}->{code};
-		return $self->{macros}->{$1}->{code}->($self, $mg, eval qq|($2)|);
-	} 
+		my $v =  $self->{macros}->{$1}->{code}->($self, $mg, eval qq|($2)|);
+		return $v;
+	}
 	return '' unless $self->{macros}->{$match}->{code};
-	return $self->{macros}->{$match}->{code}->($self, $mg);	
+	return $self->{macros}->{$match}->{code}->($self, $mg);
 }
 
 sub build_sub_code {
@@ -475,13 +484,11 @@ sub build_clearer_code {
 sub build_predicate {
 	my ($self, $mg, $name, $meta) = @_;
 	$mg->sub(qq|has_$name|)
-		->code($self->build_code($mg, $name, $self->build_predicate_code($name)))	
+		->code($self->build_code($mg, $name, $self->build_predicate_code($name)))
 		->pod(qq|has_$name will return true if $name accessor has a value.|)
 		->example(qq|\$obj->has_$name|)
 		->test(
-			($meta->{$name}->{required} || $meta->{$name}->{default} ? (
-				['ok', qq|delete \$obj->{$name}|]
-			) : ()),
+			['ok', qq|do{ delete \$obj->{$name}; 1;}|],
 			['is', qq|\$obj->has_$name|, q|''|],
 			$self->build_tests($name, $meta->{$name}, 'success'),
 			['is', qq|\$obj->has_$name|, 1],
@@ -799,7 +806,6 @@ sub build_macro_attributes {
 
 sub build_macro {
 	my ($self, $mg, $class) = @_;
-
 	my $meta = $self->{macros};
 	for my $macro (@{$class}) {
 		if ($macro->[-1] !~  m/^{/) {
@@ -808,7 +814,7 @@ sub build_macro {
 			die $@ if $@;
 			my $include_meta = $include->new($macro->[0] ? do {
 				$macro->[0] =~ s/^\[|\]$//g;
-				( eval qq|$macro->[0]| ); 
+				( eval qq|$macro->[0]| );
 			} : ())->meta;
 			$meta = {%{$meta}, %{$include_meta}};
 		} else {
@@ -1054,8 +1060,10 @@ sub build_tests {
 	} : do {
 		my %test_data;
 		map {
-			push @{$test_data{test_data_columns}}, $_;
-			$test_data{$_} = [ $self->build_test_data($meta->{$_}->{type}->[0] ? $meta->{$_}->{type}->[0] : 'Any', '', 1) ];
+			unless ($meta->{$_}->{no_success_test}) {
+				push @{$test_data{test_data_columns}}, $_;
+				$test_data{$_} = [ $self->build_test_data($meta->{$_}->{type}->[0] ? $meta->{$_}->{type}->[0] : 'Any', '', 1) ]
+			}
 		} grep { $meta->{$_}->{meta} eq 'ACCESSOR' } keys %{$meta};
 		my $valid =  join(', ', map { sprintf '%s => %s', $_, $test_data{$_}->[0] } grep { $meta->{$_}->{required} } @{$test_data{test_data_columns}});
 		push @tests, [
@@ -1132,17 +1140,17 @@ sub build_tests {
 		$meta->{private} ? do {
 			push @tests, ['eval', qq|\$obj->$name|, 'private method|private attribute'];
 		} : do {
-			push @tests, ['is', qq|\$obj->$name|, 'undef'] if !$meta->{builder} && !$meta->{required} && !$meta->{default};
+			push @tests, ['is', qq|\$obj->$name|, 'undef'] if !$meta->{no_success_test} && !$meta->{builder} && !$meta->{required} && !$meta->{default};
 			my (@test_cases) = $self->build_test_data($meta->{type}->[0] || 'Any', $name, $meta->{required} || $meta->{builder});
 			if (scalar @test_cases > 1) {
 				my $valid = shift @test_cases;
-				push @tests, ['deep', qq|\$obj->$name($valid)|, $valid];
+				push @tests, ['deep', qq|\$obj->$name($valid)|, $valid] unless $meta->{no_success_test};
 				unless ($meta->{coerce}) {
 					for (@test_cases) {
 						push @tests, ['eval', qq|\$obj->$name($_)|, 'invalid|value|type|constraint|greater|atleast' ];
 					}
 				}
-				push @tests, ['deep', qq|\$obj->$name|, $valid];
+				push @tests, ['deep', qq|\$obj->$name|, $valid] unless $meta->{no_success_test};
 			}
 		};
 	} : do {
@@ -1196,7 +1204,7 @@ Hades - Less is more, more is less!
 
 =head1 VERSION
 
-Version 0.16
+Version 0.17
 
 =cut
 
@@ -1923,7 +1931,7 @@ Used in conjunction with Dict and Tuple to specify slots that are optional and m
 
 =head2 Macros
 
-Hades has a concept of macros that allow you to write re-usable code. see L<https://metacpan.org/source/LNATION/Hades-0.16/macro-fh.hades> for an example of how to extend via macros.
+Hades has a concept of macros that allow you to write re-usable code. see L<https://metacpan.org/source/LNATION/Hades-0.17/macro-fh.hades> for an example of how to extend via macros.
 
 	macro {
 		FH [ macro => [qw/read_file write_file/], alias => { read_file => [qw/rf/], write_file => [qw/wf/] } ]
